package com.kayaposoft.www.enrico.ws.v1_0;

import java.math.BigInteger;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.apache.axis2.transport.http.HTTPConstants;

import com.kayaposoft.www.enrico.ws.v1_0.EnricoStub.*;

public class Main {

	/**
	 * @param args
	 * @throws RemoteException 
	 */
	public static void main(String[] args) throws RemoteException {

		try
		{
			EnricoStub client = new EnricoStub("http://www.kayaposoft.com/enrico/ws/v1.0/index.php");
			client._getServiceClient().getOptions().setProperty(HTTPConstants.CHUNKED, false);
			Calendar today = new GregorianCalendar();
			int day = today.get(Calendar.DAY_OF_MONTH);
			int month = today.get(Calendar.MONTH) + 1;
			int year = today.get(Calendar.YEAR);
	
			System.out.println("Public holidays in Germany (Baden-Württemberg) for " + ConvertMonth(month) + " " + year);
			GetPublicHolidaysForMonth request = new GetPublicHolidaysForMonth();
			request.setMonth(new BigInteger(Integer.toString(month)));
			request.setYear(new BigInteger(Integer.toString(year)));
			request.setCountry("ger");
			request.setRegion("Baden-Württemberg");
			GetPublicHolidaysForMonthResponse response = client.getPublicHolidaysForMonth(request);
	        if (response.get_return().getError() != null && response.get_return().getError().length() > 0)
	        {
	            PrintError(response.get_return().getError());
	        }
	        else
	        {
	            PrintPublicHolidays(response.get_return().getPublicHolidays());
	        }
			
	        System.out.println("\r\nPublic holidays in Estonia for " + year);
	        GetPublicHolidaysForYear request2 = new GetPublicHolidaysForYear();
	        request2.setYear(new BigInteger(Integer.toString(year)));
	        request2.setCountry("est");
	        request2.setRegion("");
	        GetPublicHolidaysForYearResponse response2 = client.getPublicHolidaysForYear(request2);
	        if (response2.get_return().getError() != null && response2.get_return().getError().length() > 0)
	        {
	            PrintError(response2.get_return().getError());
	        }
	        else
	        {
	            PrintPublicHolidays(response2.get_return().getPublicHolidays());
	        }
	
	        System.out.println("\r\nPublic holidays in United States of America (District Of Columbia) from 04 July " + year + " (including) to 04 July " + (year + 2) + " (including)");
	        GetPublicHolidaysForDateRange request3 = new GetPublicHolidaysForDateRange();
	        request3.setCountry("usa");
	        request3.setRegion("District Of Columbia");
	        DateType from = new DateType();
	        from.setDay(4);
	        from.setMonth(7);
	        from.setYear(year);
	        request3.setFromDate(from);
	        DateType to = new DateType();
	        to.setDay(4);
	        to.setMonth(7);
	        to.setYear(year + 2);
	        request3.setToDate(to);
	        GetPublicHolidaysForDateRangeResponse response3 = client.getPublicHolidaysForDateRange(request3);
	        if (response3.get_return().getError() != null && response3.get_return().getError().length() > 0)
	        {
	            PrintError(response3.get_return().getError());
	        }
	        else
	        {
	            PrintPublicHolidays(response3.get_return().getPublicHolidays());
	        }
	
	        System.out.println("\r\nIs " + day + " " + ConvertMonth(month) + " " + year + " public holiday in Slovakia? " );
	        DateType todayDT = new DateType();
	        todayDT.setDay(day);
	        todayDT.setMonth(month);
	        todayDT.setYear(year);
	        IsPublicHoliday request4 = new IsPublicHoliday();
	        request4.setCountry("svk");
	        request4.setDate(todayDT);
	        request4.setRegion("this is ignored by enrico when country has no regions");
	        IsPublicHolidayResponse response4 = client.isPublicHoliday(request4);
	        if (response4.get_return().getError() != null && response4.get_return().getError().length() > 0)
	        {
	            PrintError(response4.get_return().getError());
	        }
	        else
	        {
	            if (response4.get_return().getIsPublicHoliday())
	            {
	                System.out.println("YES");
	            }
	            else
	            {
	                System.out.println("NO");
	            }
	        }
	        
	        System.out.println("\r\nAll countries supported by Enrico");
	        GetSupportedCountries request5 = new GetSupportedCountries();
	        GetSupportedCountriesResponse response5 = client.getSupportedCountries(request5);
	        if (response5.get_return().getError() != null && response5.get_return().getError().length() > 0)
	        {
	            PrintError(response5.get_return().getError());
	        }
	        else
	        {
	        	SupportedCountryType[] supportedCountries = response5.get_return().getSupportedCountries();
	            if (supportedCountries == null || supportedCountries.length == 0)
	            {
	                System.out.println("NONE");
	            }
	            else
	            {
	                for (SupportedCountryType country : supportedCountries)
	                {
	                    String output = country.getFullName() + " (" + country.getCountryCode() + "), dates available from " + GetDate(country.getFromDate())
	                        + ", to " + GetDate(country.getToDate());
	
	                    String reg = "";
	                    if (country.getRegions() != null && country.getRegions().length > 0)
	                    {
	                        for (String region : country.getRegions())
	                        {
	                            if (reg.length() > 0) reg += ", ";
	                            reg += region;
	                        }
	                    }
	                    if (reg != null && reg.length() > 0)
	                    {
	                        output += ", " + country.getRegions().length + " regions (" + reg + ")";
	                    }
	
	                    System.out.println(output);
	                }
	            }
	        }
		}
		catch(Exception e)
		{
			PrintError(e.getMessage());
		}
	}
	
	private static void PrintError(String error)
    {
        System.err.println("ERROR occured: " + error);
    }

	private static void PrintPublicHolidays(PublicHolidayType[] publicHolidays)
    {
        if (publicHolidays != null)
        {
            if (publicHolidays.length == 0)
            {
                System.out.println("No holidays");
            }
            else
            {
                for (PublicHolidayType publicHoliday : publicHolidays)
                {
                    System.out.println(GetDate(publicHoliday.getDate())
                        + " (" + ConvertDayOfWeek(publicHoliday.getDate().getDayOfWeek()) + ")" + " - " + publicHoliday.getLocalName() + " (" + publicHoliday.getEnglishName() + ")");
                }
            }
        }
        else
        {
            System.out.println("No holidays");
        }
    }

	private static String ConvertMonth(int month)
    {
        switch (month)
        {
            case 1: return "Jan";
            case 2: return "Feb";
            case 3: return "Mar";
            case 4: return "Apr";
            case 5: return "May";
            case 6: return "Jun";
            case 7: return "Jul";
            case 8: return "Aug";
            case 9: return "Sep";
            case 10: return "Oct";
            case 11: return "Nov";
            case 12: return "Dec";
        }

        return "";
    }

	private static String ConvertDayOfWeek(int dayOfWeek)
    {
        switch (dayOfWeek)
        {
            case 1: return "Monday";
            case 2: return "Tuesday";
            case 3: return "Wednesday";
            case 4: return "Thursday";
            case 5: return "Friday";
            case 6: return "Saturday";
            case 7: return "Sunday";
        }

        return "";
    }
	
	private static String GetDate(DateType date)
    {
        return date.getDay() + " " + ConvertMonth(date.getMonth()) + " " + date.getYear();
    }
}
